from odoo import api, fields, models

from odoo.addons.payment_stripe import const as stripe_const


# See https://epayments-support.ingenico.com/en/get-started/transaction-management/transaction-error-codes
# or through the BO: Support > User Guides > List of the payment statuses and error codes
PAYMENT_ERROR_TYPES_MAPPING = {
    30331001: 'card-expired',
    30431001: 'card-stolen',
    30541001: 'card-expired',
}


class PaymentTransaction(models.Model):
    _inherit = 'payment.transaction'

    ogone_error_code = fields.Integer('Ogone Error Code (NCERROR)', readonly=True)

    @api.model_create_multi
    def create(self, vals_list):
        transactions = super().create(vals_list)
        for tx in transactions:
            if tx.tokenize and tx.payment_method_id.code == "bancontact":
                # Nah! bancontact does not support tokenization so never request it
                tx.tokenize = False
        return transactions

    def _ogone_payment_error_type(self):
        """Return the error type related to the Ogone Error Code"""
        self.ensure_one()
        return PAYMENT_ERROR_TYPES_MAPPING.get(self.ogone_error_code, '')

    def _process_notification_data(self, notification_data):
        """ Override of payment to process the transaction based on Ogone data and auto-confirm ACH Stripe transactions
        for Odoo Inc.

        Note: self.ensure_one()

        :param dict notification_data: The notification data sent by the provider
        :return: None
        """
        # For microdeposit transactions Stripe returns status requires_action. Transactions
        # authenticated through the bank are processing/pending.
        auto_confirm_status = stripe_const.STATUS_MAPPING['draft'] + stripe_const.STATUS_MAPPING['pending']
        if self.provider_code == 'stripe' and \
           self.company_id == self.env.ref('openerp_enterprise.company_us') and \
           self.payment_method_code == 'ach_direct_debit' and \
           self.state == 'draft' and \
           notification_data.get('payment_intent', {}).get('status') in auto_confirm_status:
            notification_data['payment_intent']['status'] = 'succeeded'

        super()._process_notification_data(notification_data)

        if 'tree' in notification_data:
            notification_data = notification_data['tree']
        if self.provider_code == "ogone" and self.state == "error" and notification_data.get("NCERROR"):
            # store Ogone transaction error code for later use
            self.ogone_error_code = int(notification_data.get('NCERROR', '0'))

    def _create_payment(self):
        # to accomodate for cases where the payment token is on a different
        # company than the invoices (ex. Odoo DMCC, Odoo KE, ...), we need to
        # ensure that we allow both company of the payment and companies from the
        # invoices.
        allowed_company_ids = self.env.context.get('allowed_company_ids', [])
        missing_company_ids = set(self.invoice_ids.sudo().company_id.ids) - set(allowed_company_ids)
        self = self.with_context(allowed_company_ids=allowed_company_ids + list(missing_company_ids))
        return super()._create_payment()
